/**
 * DREAM Copyright (C) 2003 2004 INRIA Rhone-Alpes
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Contact: dream@objectweb.org
 * 
 * Initial developer(s): Matthieu Leclercq Contributor(s):
 */

import org.objectweb.dream.aaa.AgentServer;
import org.objectweb.dream.aaa.agent.Agent;
import org.objectweb.dream.aaa.agent.AgentId;
import org.objectweb.dream.aaa.agent.Notification;
import org.objectweb.dream.aaa.agent.PushNotification;
import org.objectweb.fractal.api.Component;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Start an agentserver, deploy a {@link TestAgent}a send it a
 * {@link Notification}.
 */
public final class Main
{

  private static class A0 extends Agent
  {

    protected AgentId a1;
    protected AgentId a2;

    /**
     * @see Agent#Agent(short, short)
     */
    public A0(short local, short to)
    {
      super(local, to, "A0");
    }

    /**
     * @see Agent#react(AgentId, Notification)
     */
    public void react(AgentId from, Notification not) throws Exception
    {
      logger.log(BasicLevel.INFO, "has received a notification from " + from);
      sendTo(a1, new Notification());
      sendTo(a2, new Notification());
    }
  }

  private static class A1 extends Agent
  {

    protected AgentId a2;

    /**
     * @see Agent#Agent(short, short)
     */
    public A1(short local, short to)
    {
      super(local, to, "A1");
    }

    /**
     * @see Agent#react(AgentId, Notification)
     */
    public void react(AgentId from, Notification not) throws Exception
    {
      logger.log(BasicLevel.INFO, "has received a notification from " + from);
      sendTo(a2, new Notification());
    }
  }

  private static class A2 extends Agent
  {

    /**
     * @see Agent#Agent(short, short)
     */
    public A2(short local, short to)
    {
      super(local, to, "A2");
    }

    /**
     * @see Agent#react(AgentId, Notification)
     */
    public void react(AgentId from, Notification not) throws Exception
    {
      logger.log(BasicLevel.INFO, "has received a notification from " + from);
    }
  }

  private Main()
  {
  }

  /**
   * Entry point
   * 
   * @param args the server configuration file
   * @throws Exception if an error occurs
   */
  public static void main(String[] args) throws Exception
  {
    Component s0 = AgentServer.createAgentServer(args[0], "S0");
    Component s1 = AgentServer.createAgentServer(args[0], "S1");
    Component s2 = AgentServer.createAgentServer(args[0], "S2");
    PushNotification pn0 = AgentServer.getPushNotificationItf(s0);

    short r0 = AgentServer.getDefaultRepositoryId(s0);
    short r1 = AgentServer.getDefaultRepositoryId(s1);
    short r2 = AgentServer.getDefaultRepositoryId(s2);

    A0 a0 = new A0(r0, r0);
    A1 a1 = new A1(r0, r1);
    A2 a2 = new A2(r0, r2);

    a0.a1 = a1.getId();
    a0.a2 = a2.getId();
    a1.a2 = a2.getId();

    a0.deploy(pn0);
    a1.deploy(pn0);
    a2.deploy(pn0);

    while (true)
    {
      pn0.pushNotification(null, a0.getId(), new Notification());
      Thread.sleep(2000);
    }
  }
}