/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.aaa.components.conf.AgentServerConfiguration;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.router.AbstractKeyBasedRouterImpl;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * The notification routers routes notifications received on its push input to
 * one of its outputs. These outputs are bound to components implementing the
 * {@link AccessibleRepositories}interface.
 */
public class AAAMessageRouterImpl extends AbstractKeyBasedRouterImpl
{

  /**
   * The {@link AgentServerConfiguration }client interface of this component.
   */
  protected AgentServerConfiguration agentServerConfigurationItf;

  /**
   * @see AbstractKeyBasedRouterImpl#getOutputKey(Message)
   */
  protected Object getOutputKey(Message message)
  {
    AgentChunk agentChunk = (AgentChunk) message
        .getChunk(AgentChunk.DEFAULT_NAME);
    if (agentChunk == null)
    {
      logger.log(BasicLevel.ERROR, "Unable to find agent chunk named "
          + AgentChunk.DEFAULT_NAME + ". The message is dropped");
      return null;
    }
    return new Short(agentChunk.getAgentTo().getTo());
  }

  /**
   * @see AbstractKeyBasedRouterImpl#initRouteTable()
   */
  protected void initRouteTable() throws PushException
  {
    if (initialized)
    {
      return;
    }
    defaultOutPushItf = null;
    outPushRouteTable = new Hashtable();

    int length = 0;
    int i = 0;
    short[][] regs = new short[outPushMap.size()][];
    Iterator iter = outPushMap.entrySet().iterator();
    while (iter.hasNext())
    {
      Map.Entry entry = (Map.Entry) iter.next();
      String name = ((String) entry.getKey()).substring(OUT_PUSH_ITF_NAME
          .length());
      Push associatedPushItf = (Push) entry.getValue();
      short[] accessibleReg = agentServerConfigurationItf
          .getAccessibleRepositories(name);
      regs[i] = accessibleReg;
      if (accessibleReg == null)
      {
        defaultOutPushItf = associatedPushItf;
      }
      else
      {
        length += regs[i].length;
        for (int j = 0; j < accessibleReg.length; j++)
        {
          outPushRouteTable.put(new Short(accessibleReg[j]), associatedPushItf);
        }
      }
      i++;
    }
    initialized = true;
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    String[] list = super.listFc();
    String[] result = new String[list.length + 1];
    result[0] = AgentServerConfiguration.DEFAULT_NAME;
    System.arraycopy(list, 0, result, 1, list.length);
    return result;
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.startsWith(AgentServerConfiguration.DEFAULT_NAME))
    {
      agentServerConfigurationItf = (AgentServerConfiguration) serverItf;
    }
  }
}