/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components;

import org.objectweb.dream.aaa.agent.AgentId;
import org.objectweb.dream.aaa.agent.Notification;
import org.objectweb.dream.message.BasicChunkType;
import org.objectweb.dream.message.ChunkType;

/**
 * An interface for chunk containing a notification, a sender agentId and a
 * receiver agentId.
 */
public interface AgentChunk
{

  /** The default name for chunks of this type */
  String    DEFAULT_NAME = "agentChunk";

  /** The chunk type of chunk implementing this interface */
  ChunkType TYPE         = new BasicChunkType(AgentChunk.class,
                             BasicAgentChunk.class);

  /**
   * Returns the identifier of the sender agent.
   * 
   * @return the identifier of the sender agent.
   */
  AgentId getAgentFrom();

  /**
   * Sets the identifier of the sender agent.
   * 
   * @param id the identifier of the sender agent.
   */
  void setAgentFrom(AgentId id);

  /**
   * Returns the identifier of the receiver agent.
   * 
   * @return the identifier of the receiver agent.
   */
  AgentId getAgentTo();

  /**
   * Sets the identifier of the receiver agent.
   * 
   * @param id the identifier of the receiver agent.
   */
  void setAgentTo(AgentId id);

  /**
   * Returns the notification.
   * 
   * @return the notification.
   */
  Notification getAgentNotification();

  /**
   * Sets the notification.
   * 
   * @param not the notification.
   */
  void setAgentNotifiaction(Notification not);

}