/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.aaa.agent.AgentId;
import org.objectweb.dream.aaa.agent.Notification;
import org.objectweb.dream.aaa.agent.PushNotification;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Simple component that create a message containing a {@link AgentChunk }chunk
 * with the given informations
 */
public class NotifToMessageImpl extends AbstractComponent
    implements
      PushNotification
{

  private Push           outPushItf;
  private MessageManager messageManagerItf;

  /**
   * @see PushNotification#pushNotification(AgentId, AgentId, Notification)
   */
  public void pushNotification(AgentId from, AgentId to, Notification not)
  {
    if (logger.isLoggable(BasicLevel.DEBUG))
    {
      logger.log(BasicLevel.DEBUG, "pushNotification from=" + from + ", to="
          + to + " notification " + not);
    }
    Message message = null;
    message = messageManagerItf.createMessage(AAAMessage.TYPE);

    AgentChunk agentChunk = (AgentChunk) message
        .getChunk(AgentChunk.DEFAULT_NAME);
    if (agentChunk == null)
    {
      if (!(message instanceof ExtensibleMessage))
      {
        throw new InternalError("Unextensible mesasge without agent chunk");
      }
    }
    if (from != null)
    {
      agentChunk.setAgentFrom(from);
    }
    else
    {
      agentChunk.setAgentFrom(AgentId.NULL_ID);
    }
    if (to != null)
    {
      agentChunk.setAgentTo(to);
    }
    else
    {
      agentChunk.setAgentTo(AgentId.NULL_ID);
    }
    agentChunk.setAgentNotifiaction(((Notification) not.clone()));
    try
    {
      if (logger.isLoggable(BasicLevel.DEBUG))
      {
        logger.log(BasicLevel.DEBUG, "push message " + message);
      }
      outPushItf.push(message, null);
    }
    catch (PushException e)
    {
      logger.log(BasicLevel.ERROR, "Error in push of message", e);
      messageManagerItf.deleteMessage(message);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
    super.bindFc(clientItfName, serverItf);
  }

}