/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components.conf;

/**
 * Repository description
 */
public final class RepositoryDesc
{

  /** The name of the repository. */
  protected String     name;

  /** The name of the engine ADL for this repository */
  protected String     engineADLName;

  /** The repository ID */
  protected short      repositoryID;

  /** The server description where this repository is deployed */
  protected ServerDesc server;

  /**
   * Constructor
   * 
   * @param name the name of this repository.
   * @param engineADLName The name of the engine ADL for this repository
   * @param repositoryID the repository ID.
   * @param server the server description where this repository is deployed.
   */
  public RepositoryDesc(String name, String engineADLName, short repositoryID,
      ServerDesc server)
  {
    this.name = name;
    this.engineADLName = engineADLName;
    this.repositoryID = repositoryID;
    this.server = server;
  }

  /**
   * Returns the repository ID
   * 
   * @return the repository ID
   */
  public short getRepositoryID()
  {
    return repositoryID;
  }

  /**
   * Returns the server description where this repository is deployed
   * 
   * @return the server description where this repository is deployed
   */
  public ServerDesc getServer()
  {
    return server;
  }

  /**
   * returns the name of the repository.
   * 
   * @return the name of the repository.
   */
  public String getName()
  {
    return name;
  }

  /**
   * Returns the name of the engine ADL for this repository
   * 
   * @return The name of the engine ADL for this repository
   */
  public String getEngineADLName()
  {
    return engineADLName;
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("{name=").append(name);
    sb.append(", id=").append(repositoryID);
    sb.append("}");
    return sb.toString();
  }

}