/**
 * Copyright (C) 2002 
 */

package org.objectweb.util.monolog.api;

/**
 * It permits to manage Handler instances.
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface HandlerFactory {

	/**
	 * It retrieves a new instance of an handler which the type is specified
	 * by the parameter.
	 * @param 	handlertype is the type of the parameter. The possible value are
	 * 			defined in this interface by the XXX_HANDLER_TYPE constants.
	 * @return	a new instance of an handler
	 */
	Handler createHandler(String hn, byte handlertype);

	/**
	 * It retrieves all handler managed by this factory.
	 * @return a set of Handler instance or an empty set.
	 */
	Handler[] getHandlers();

	/**
	 * It retrieves the handler which the name is specified by the parameter
	 * @param handlername is the name of the handler
	 * @return an handler instance or a null value.
	 */
	Handler getHandler(String handlername);

	/**
	 * It removes the handler which the name is specified by the parameter
	 * @param handlername is the name of the handler
	 * @return the removed handler instance or a null value if it does not
	 * exist.
	 */
	Handler removeHandler(String handlername);
}
