/**
 * @Contributors:<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>S. Chassande-Barrioz</b></A>
 */

package org.objectweb.util.monolog.api;

/**
 * <P>The aims of this interface is to provide Logger instances, to allow 
 * instrumentation to be independent of the logging implementation and
 * to allow the use of the same logging implementation for all the 
 * components of a given application.</P>
 *
 * <P>This interface provides two methods to fetch Logger. If the Logger 
 * described by the parameters does not exist, then the LoggerFactory
 * must return a new instance of Logger.</P>
 * 
 * <P>The LoggerFactory interface also provides accessors to a resource 
 * bundle name associated to a LoggerFactory. This interface allows defining
 * a resource bundle name used by all Logger.</P>
 */
public interface LoggerFactory {
	
	/**
	 * This method permits to fetch a Logger. If the Logger 
	 * described by the parameters does not exist, then the LoggerFactory
	 * must return a new instance of Logger.
	 * The key parameter is a description of the expected Logger. In simple
	 * cases, the key is the initial topic of the Logger.
	 */
	Logger getLogger(String key);
	
	/**
	 * This method permits to fetch a Logger. If the Logger 
	 * described by the parameters does not exist, then the LoggerFactory
	 * must return a new instance of Logger.
	 * The key parameter is a description of the expected Logger. In simple
	 * cases, the key is the initial topic of the Logger.
	 * The resourceBundleName parameter allows specifying the name of a 
	 * resource bundle in order to internationalise the logging. 
	 */
	Logger getLogger(String key, String resourceBundleName);
	
	/**
	 * Accessors to a resource bundle name associated to a LoggerFactory.
	 */
	String getResourceBundleName();
	
	/**
	 * Accessors to a resource bundle name associated to a LoggerFactory.
	 */
	void setResourceBundleName(String resourceBundleName);

	/**
	 * It retrieves a list of all loggers.
	 */
	Logger[] getLoggers();
}
